// MIT © 2017 azu
"use strict";

var _captalize = require("./captalize");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _require = require("textlint-rule-helper"),
    RuleHelper = _require.RuleHelper;

var StringSource = require("textlint-util-to-string");

var _require2 = require("sentence-splitter"),
    split = _require2.split,
    SentenceSyntax = _require2.Syntax;

var shouldNotCapitalized = function shouldNotCapitalized(string) {
    // A quotation
    if (!/^\w/.test(string)) {
        return true;
    }
    // proper word
    var pos = (0, _captalize.getPosFromSingleWord)(string);
    if (/^NNP/.test(pos)) {
        return true;
    }
    return false;
};

/**
 * get node at index
 * @param node
 * @param index
 * @returns {undefined|*}
 */
var getNodeAtIndex = function getNodeAtIndex(node, index) {
    if (!node || !node.children) {
        return undefined;
    }
    for (var i = 0; i < node.children.length; i++) {
        var childNode = node.children[i];
        if (childNode.range[0] <= index && index <= childNode.range[1]) {
            return childNode;
        }
    }
    return undefined;
};
/**
 * @param node
 * @param Syntax
 * @param {function} getSource
 * @param report
 * @param RuleError
 * @param fixer
 * @param {boolean} figures enable figures check
 */
var checkNode = function checkNode(_ref) {
    var node = _ref.node,
        Syntax = _ref.Syntax,
        getSource = _ref.getSource,
        report = _ref.report,
        RuleError = _ref.RuleError,
        fixer = _ref.fixer,
        figures = _ref.figures;

    var source = new StringSource(node);
    var sourceText = source.toString();
    var sentences = split(sourceText);
    sentences.filter(function (sentence) {
        return sentence.type === SentenceSyntax.Sentence;
    }).forEach(function (sentence) {
        var originalIndex = source.originalIndexFromIndex(sentence.range[0]);
        var targetNode = void 0;
        if (node.type === Syntax.ListItem) {
            targetNode = getNodeAtIndex(node.children[0], node.range[0] + originalIndex);
        } else {
            targetNode = getNodeAtIndex(node, originalIndex);
        }
        if (!targetNode) {
            return;
        }
        var DocumentURL = "https://owl.english.purdue.edu/owl/resource/592/01/";
        // check
        if (targetNode.type === Syntax.Str) {
            var text = sentence.value;
            var firstWord = text.split(" ")[0];
            if ((0, _captalize.isCapitalized)(firstWord) || shouldNotCapitalized(firstWord)) {
                return;
            }
            var index = originalIndex;
            return report(node, new RuleError("Heading: Follow the standard capitalization rules for American English.\nSee " + DocumentURL, {
                index: index,
                fix: fixer.replaceTextRange([index, index + firstWord.length], (0, _captalize.upperFirstCharacter)(firstWord))
            }));
        } else if (figures && targetNode.type === Syntax.Image && typeof targetNode.alt === "string") {
            var _text = targetNode.alt;
            if ((0, _captalize.isCapitalized)(_text) || shouldNotCapitalized(_text)) {
                return;
            }
            return report(targetNode, new RuleError("Image alt: Follow the standard capitalization rules for American English\nSee " + DocumentURL));
        }
    });
};

var DefaultOptions = {
    heading: true,
    figures: true,
    lists: true
};
var report = function report(context) {
    var _ref2;

    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var Syntax = context.Syntax,
        RuleError = context.RuleError,
        getSource = context.getSource,
        fixer = context.fixer,
        report = context.report;

    var heading = options.heading !== undefined ? options.heading : DefaultOptions.heading;
    var figures = options.figures !== undefined ? options.figures : DefaultOptions.figures;
    var lists = options.lists !== undefined ? options.lists : DefaultOptions.lists;
    var helper = new RuleHelper(context);
    return _ref2 = {}, _defineProperty(_ref2, Syntax.Header, function (node) {
        // options
        if (!heading) {
            return;
        }
        checkNode({ node: node, Syntax: Syntax, getSource: getSource, report: report, RuleError: RuleError, fixer: fixer, figures: figures });
    }), _defineProperty(_ref2, Syntax.Paragraph, function (node) {
        if (helper.isChildNode(node, [Syntax.Link, Syntax.Image, Syntax.BlockQuote, Syntax.Emphasis])) {
            return;
        }
        if (helper.isChildNode(node, [Syntax.ListItem])) {
            return;
        }
        checkNode({ node: node, Syntax: Syntax, getSource: getSource, report: report, RuleError: RuleError, fixer: fixer, figures: figures });
    }), _defineProperty(_ref2, Syntax.ListItem, function (node) {
        if (!lists) {
            return;
        }
        checkNode({ node: node, Syntax: Syntax, getSource: getSource, report: report, RuleError: RuleError, fixer: fixer, figures: figures });
    }), _ref2;
};
module.exports = {
    linter: report,
    fixer: report
};
//# sourceMappingURL=textlint-rule-en-capitalization.js.map